<?php

namespace Drupal\filter_empty_tags\Plugin\Filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\filter\Annotation\Filter;
use Drupal\filter\FilterProcessResult;
use Drupal\filter\Plugin\FilterBase;

/**
 * @Filter(
 *   id = "filter_empty_tags",
 *   title = @Translation("Filter Empty Tags"),
 *   description = @Translation("Recursively remove empty tags."),
 *   type = Drupal\filter\Plugin\FilterInterface::TYPE_TRANSFORM_IRREVERSIBLE,
 *   settings = {
 *     "do_not_consider_empty" = "button canvas drupal-media drupal-entity iframe object script svg textarea td th",
 *     "filter_spaces" = TRUE,
 *     "filter_nbsp" = TRUE,
 *     "filter_br" = TRUE,
 *   }
 * )
 */
class FilterEmptyTags extends FilterBase {

  /**
   * Static cache of regex pattern used during processing.
   *
   * @var string
   */
  protected $pattern;

  public function settingsForm(array $form, FormStateInterface $form_state) {
    $form['do_not_consider_empty'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Non filterable tags'),
      '#default_value' => $this->settings['do_not_consider_empty'],
      '#description' => $this->t('List of tags to not consider empty, hence to not filter.'),
      '#attached' => [
        'library' => [
          'filter/drupal.filter.filter_html.admin',
        ],
      ],
    ];
    $form['filter_spaces'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Consider tags with only spaces as empty'),
      '#default_value' => $this->settings['filter_spaces'],
    ];
    $form['filter_nbsp'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Consider tags with only non-breakable spaces (nbsp) as empty'),
      '#default_value' => $this->settings['filter_nbsp'],
    ];
    $form['filter_br'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Consider tags with only &lt;br&gt; as empty'),
      '#default_value' => $this->settings['filter_br'],
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function setConfiguration(array $configuration) {
    if (isset($configuration['settings']['do_not_consider_empty'])) {
      // The javascript in core/modules/filter/filter.filter_html.admin.js
      // removes new lines and double spaces so, for consistency when javascript
      // is disabled, remove them.
      $configuration['settings']['do_not_consider_empty'] = preg_replace('/\s+/', ' ', $configuration['settings']['do_not_consider_empty']);
    }

    parent::setConfiguration($configuration);
  }

  /**
   * Get the regex pattern used for processing.
   *
   * @return string
   */
  protected function getPattern() {
    if (!isset($this->pattern)) {
      $filters = [];
      if ($this->configuration['settings']['filter_spaces']) {
        $filters[] = '\s';
      }
      if ($this->configuration['settings']['filter_nbsp']) {
        $filters[] = '&nbsp;|\xc2\xa0';
      }
      if ($this->configuration['settings']['filter_br']) {
        $filters[] = '<br>|<br\/>|<br \/>';
      }
      $this->pattern = '/<([^<\/>)]*)\b[^>]*>((' . implode('|', $filters) . ')*?)<\/\1>/imsU';
    }
    return $this->pattern;
  }

  /**
   * {@inheritdoc}
   */
  public function process($text, $langcode) {
    $doNotConsiderEmpty = explode(' ', $this->settings['do_not_consider_empty']);

    // A tag is considered empty if it contains only whitespace,
    // non-breaking spaces, and break tags (per configured settings).
    $processed = $text;
    $matches = [];
    preg_match_all(
      $this->getPattern(),
      $text,
      $matches,
    );

    foreach ($matches[1] as $key => $tag) {
      if (!in_array($tag, $doNotConsiderEmpty, TRUE)) {
        $processed = str_replace($matches[0][$key], '', $processed);
      }
    }

    // If str_replace made any changes, we want to process again to handle
    // recursive structures of empty tags. This allows for more complicated
    // patterns than using the regex (?R) construct.
    if ($processed === $text) {
      return new FilterProcessResult($text);
    }
    return $this->process($processed, $langcode);
  }

}
